@@ -63,6 +63,7 @@ static void _weather_internal_check (void);
 
 enum {
     PROP_0,
+    PROP_WORLD,
     PROP_LOCATION,
     PROP_TYPE,
     PROP_LAST
@@ -544,10 +545,11 @@ gweather_info_finalize (GObject *object)
     _weather_location_free (priv->location);
     priv->location = NULL;
 
-    if (priv->glocation) {
+    if (priv->glocation)
 	gweather_location_unref (priv->glocation);
-	priv->glocation = NULL;
-    }
+
+    if (priv->world)
+	gweather_location_unref (priv->world);
 
     g_free (priv->forecast);
     priv->forecast = NULL;
@@ -787,26 +789,21 @@ windspeed_string (gfloat knots, GWeatherSpeedUnit to_unit)
     switch (to_unit) {
     case GWEATHER_SPEED_UNIT_KNOTS:
 	/* TRANSLATOR: This is the wind speed in knots */
-	g_strdup_printf(_("%0.1f knots"), knots);
-	break;
+	return g_strdup_printf(_("%0.1f knots"), knots);
     case GWEATHER_SPEED_UNIT_MPH:
 	/* TRANSLATOR: This is the wind speed in miles per hour */
-	g_strdup_printf(_("%.1f mph"), WINDSPEED_KNOTS_TO_MPH (knots));
-	break;
+	return g_strdup_printf(_("%.1f mph"), WINDSPEED_KNOTS_TO_MPH (knots));
     case GWEATHER_SPEED_UNIT_KPH:
 	/* TRANSLATOR: This is the wind speed in kilometers per hour */
-	g_strdup_printf(_("%.1f km/h"), WINDSPEED_KNOTS_TO_KPH (knots));
-	break;
+	return g_strdup_printf(_("%.1f km/h"), WINDSPEED_KNOTS_TO_KPH (knots));
     case GWEATHER_SPEED_UNIT_MS:
 	/* TRANSLATOR: This is the wind speed in meters per second */
-	g_strdup_printf(_("%.1f m/s"), WINDSPEED_KNOTS_TO_MS (knots));
-	break;
+	return g_strdup_printf(_("%.1f m/s"), WINDSPEED_KNOTS_TO_MS (knots));
     case GWEATHER_SPEED_UNIT_BFT:
 	/* TRANSLATOR: This is the wind speed as a Beaufort force factor
 	 * (commonly used in nautical wind estimation).
 	 */
-	g_strdup_printf(_("Beaufort force %.1f"), WINDSPEED_KNOTS_TO_BFT (knots));
-	break;
+	return g_strdup_printf(_("Beaufort force %.1f"), WINDSPEED_KNOTS_TO_BFT (knots));
     case GWEATHER_SPEED_UNIT_INVALID:
     case GWEATHER_SPEED_UNIT_DEFAULT:
     default:
@@ -863,28 +860,22 @@ gweather_info_get_pressure (GWeatherInfo *info)
     switch (g_settings_get_enum (priv->settings, PRESSURE_UNIT)) {
     case GWEATHER_PRESSURE_UNIT_INCH_HG:
 	/* TRANSLATOR: This is pressure in inches of mercury */
-	g_strdup_printf(_("%.2f inHg"), priv->pressure);
-	break;
+	return g_strdup_printf(_("%.2f inHg"), priv->pressure);
     case GWEATHER_PRESSURE_UNIT_MM_HG:
 	/* TRANSLATOR: This is pressure in millimeters of mercury */
-	g_strdup_printf(_("%.1f mmHg"), PRESSURE_INCH_TO_MM (priv->pressure));
-	break;
+	return g_strdup_printf(_("%.1f mmHg"), PRESSURE_INCH_TO_MM (priv->pressure));
     case GWEATHER_PRESSURE_UNIT_KPA:
 	/* TRANSLATOR: This is pressure in kiloPascals */
-	g_strdup_printf(_("%.2f kPa"), PRESSURE_INCH_TO_KPA (priv->pressure));
-	break;
+	return g_strdup_printf(_("%.2f kPa"), PRESSURE_INCH_TO_KPA (priv->pressure));
     case GWEATHER_PRESSURE_UNIT_HPA:
 	/* TRANSLATOR: This is pressure in hectoPascals */
-	g_strdup_printf(_("%.2f hPa"), PRESSURE_INCH_TO_HPA (priv->pressure));
-	break;
+	return g_strdup_printf(_("%.2f hPa"), PRESSURE_INCH_TO_HPA (priv->pressure));
     case GWEATHER_PRESSURE_UNIT_MB:
 	/* TRANSLATOR: This is pressure in millibars */
-	g_strdup_printf(_("%.2f mb"), PRESSURE_INCH_TO_MB (priv->pressure));
-	break;
+	return g_strdup_printf(_("%.2f mb"), PRESSURE_INCH_TO_MB (priv->pressure));
     case GWEATHER_PRESSURE_UNIT_ATM:
 	/* TRANSLATOR: This is pressure in atmospheres */
-	g_strdup_printf(_("%.3f atm"), PRESSURE_INCH_TO_ATM (priv->pressure));
-	break;
+	return g_strdup_printf(_("%.3f atm"), PRESSURE_INCH_TO_ATM (priv->pressure));
 
     case GWEATHER_PRESSURE_UNIT_INVALID:
     case GWEATHER_PRESSURE_UNIT_DEFAULT:
@@ -912,16 +903,13 @@ gweather_info_get_visibility (GWeatherInfo *info)
     switch (g_settings_get_enum (priv->settings, DISTANCE_UNIT)) {
     case GWEATHER_DISTANCE_UNIT_MILES:
 	/* TRANSLATOR: This is the visibility in miles */
-	g_strdup_printf (_("%.1f miles"), priv->visibility);
-	break;
+	return g_strdup_printf (_("%.1f miles"), priv->visibility);
     case GWEATHER_DISTANCE_UNIT_KM:
 	/* TRANSLATOR: This is the visibility in kilometers */
-	g_strdup_printf (_("%.1f km"), VISIBILITY_SM_TO_KM (priv->visibility));
-	break;
+	return g_strdup_printf (_("%.1f km"), VISIBILITY_SM_TO_KM (priv->visibility));
     case GWEATHER_DISTANCE_UNIT_METERS:
 	/* TRANSLATOR: This is the visibility in meters */
-	g_strdup_printf (_("%.0fm"), VISIBILITY_SM_TO_M (priv->visibility));
-	break;
+	return g_strdup_printf (_("%.0fm"), VISIBILITY_SM_TO_M (priv->visibility));
 
     case GWEATHER_DISTANCE_UNIT_INVALID:
     case GWEATHER_DISTANCE_UNIT_DEFAULT:
@@ -1549,7 +1537,7 @@ gweather_info_get_value_apparent (GWeatherInfo *info, GWeatherTemperatureUnit un
 /**
  * gweather_info_get_value_update:
  * @info: a #GWeatherInfo
- * @value: (out) (type gulong): the time @info was last updated
+ * @value: (out) (type glong): the time @info was last updated
  *
  * Returns: TRUE is @value is valid, FALSE otherwise.
  */
@@ -1758,6 +1746,9 @@ gweather_info_set_location_internal (GWeatherInfo     *info,
     if (priv->location)
 	_weather_location_free (priv->location);
 
+    if (!priv->world)
+	priv->world = gweather_location_new_world (FALSE);
+
     priv->glocation = location;
     if (priv->glocation) {
 	gweather_location_ref (location);
@@ -1770,7 +1761,7 @@ gweather_info_set_location_internal (GWeatherInfo     *info,
 	if (strcmp(name, "") == 0)
 	    name = NULL;
 
-	priv->glocation = gweather_location_find_by_station_code (station_code);
+	priv->glocation = gweather_location_find_by_station_code (priv->world, station_code);
     }
 
     priv->location = _weather_location_from_gweather_location (priv->glocation, name);
@@ -1811,6 +1802,9 @@ gweather_info_set_property (GObject *object,
     GWeatherInfoPrivate *priv = self->priv;
 
     switch (property_id) {
+    case PROP_WORLD:
+	priv->world = g_value_dup_boxed (value);
+	break;
     case PROP_LOCATION:
 	gweather_info_set_location_internal (self, (GWeatherLocation*) g_value_get_boxed (value));
 	break;
@@ -1833,6 +1827,13 @@ gweather_info_class_init (GWeatherInfoClass *klass)
     gobject_class->finalize = gweather_info_finalize;
     gobject_class->set_property = gweather_info_set_property;
 
+    pspec = g_param_spec_boxed ("world",
+				"World",
+				"The hierarchy of locations containing the desired location",
+				GWEATHER_TYPE_LOCATION,
+				G_PARAM_STATIC_STRINGS | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
+    g_object_class_install_property (gobject_class, PROP_WORLD, pspec);
+
     pspec = g_param_spec_boxed ("location",
 				"Location",
 				"The location this info represents",
@@ -1863,6 +1864,10 @@ gweather_info_class_init (GWeatherInfoClass *klass)
  * @location: (allow-none): the desidered #GWeatherLocation (NULL for default)
  * @forecast_type: the type of forecast requested
  *
+ * Builds a new #GWeatherInfo that will provide weather information about
+ * @location. The returned info will not be ready until the ::updated signal
+ * is emitted.
+ *
  * Returns: (transfer full): a new #GWeatherInfo
  */
 GWeatherInfo *
@@ -1880,6 +1885,34 @@ gweather_info_new (GWeatherLocation    *location,
     return self;
 }
 
+/**
+ * gweather_info_new_for_world:
+ * @world: a #GWeatherLocation representing the whole world
+ * @location: (allow-none): the desidered #GWeatherLocation (NULL for default)
+ * @forecast_type: the type of forecast requested
+ *
+ * Similar to g_weather_info_new(), but also has a @world parameter, that allow controlling
+ * the hierarchy of #GWeatherLocation to which @location (or the default one taken from
+ * GSettings) belongs.
+ *
+ * Returns: (transfer full): a new #GWeatherInfo
+ */
+GWeatherInfo *
+gweather_info_new_for_world (GWeatherLocation    *world,
+			     GWeatherLocation    *location,
+			     GWeatherForecastType forecast_type)
+{
+    GWeatherInfo *self;
+
+    if (location != NULL)
+	self = g_object_new (GWEATHER_TYPE_INFO, "world", world, "location", location, "forecast-type", forecast_type, NULL);
+    else
+	self = g_object_new (GWEATHER_TYPE_INFO, "world", world, "forecast-type", forecast_type, NULL);
+    gweather_info_update (self);
+
+    return self;
+}
+
 GWeatherInfo *
 _gweather_info_new_clone (GWeatherInfo *other)
 {
